#include "BaseObj.h"

namespace ehs
{
	BaseObj::~BaseObj()
	{
	}

    BaseObj::BaseObj()
        : parent(nullptr)
    {
        AddType("BaseObj");
    }

    BaseObj::BaseObj(const BaseObj& base)
        : hierarchy(base.hierarchy), parent(nullptr)
    {
    }

    BaseObj::BaseObj(BaseObj&& base) noexcept
        : hierarchy(std::move(base.hierarchy)), parent(base.parent)
    {
        base.hierarchy = Array<Type>();
        base.parent = nullptr;
    }

    BaseObj& BaseObj::operator=(const BaseObj& base)
    {
        if (this == &base)
            return *this;

		hierarchy = base.hierarchy;
        parent = nullptr;

        return *this;
    }

    BaseObj& BaseObj::operator=(BaseObj&& base) noexcept
    {
        if (this == &base)
            return *this;

        hierarchy = std::move(base.hierarchy);
        parent = base.parent;

        base.hierarchy = Array<Type>();
        base.parent = nullptr;

        return *this;
    }

    bool BaseObj::operator!() const
    {
        return IsValid();
    }

    bool BaseObj::operator==(const BaseObj& base) const
    {
        return hierarchy == base.hierarchy;
    }

    bool BaseObj::operator!=(const BaseObj& base) const
    {
        return hierarchy != base.hierarchy;
    }

    bool BaseObj::HasType(const Str_8& type) const
    {
        return HasType(type.Hash_64());
    }

    bool BaseObj::AddType(Str_8 type)
    {
        if (HasType(type))
            return false;

		hierarchy.Push(std::move(type));

        return true;
    }

    Type BaseObj::GetType() const
    {
        return hierarchy[hierarchy.End()];
    }

    Type BaseObj::GetType(const Str_8& type) const
    {
        return GetType(type.Hash_64());
    }

    const Char_8* BaseObj::GetTypeId() const
    {
        return GetType().GetId();
    }

	UInt_64 BaseObj::GetTypeHashId() const
    {
        return GetType().GetHashId();
    }

    const Array<Type>& BaseObj::GetHierarchy() const
    {
        return hierarchy;
    }

    void BaseObj::SetParent(BaseObj* newParent)
    {
        parent = newParent;
    }

    BaseObj* BaseObj::GetParent()
    {
        return parent;
    }

    BaseObj* BaseObj::GetParent(const Str_8& type)
    {
        return GetParent(type.Hash_64());
    }

    BaseObj* BaseObj::Clone() const
    {
		return new BaseObj(*this);
    }

    bool BaseObj::IsValid() const
    {
        return hierarchy.Size();
    }
}